# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from .authorize import AuthorizeEndpoint
from .clientredirection import ClientRedirectionEndpoint
from .metadata import MetadataEndpoint
from .token import TokenEndpoint
from .upstreamauthorization import UpstreamAuthorizationEndpoint


__all__: list[str] = [
    'AuthorizeEndpoint',
    'ClientRedirectionEndpoint',
    'MetadataEndpoint',
    'TokenEndpoint',
    'UpstreamAuthorizationEndpoint',
]