# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import asyncio

import google.auth
from google.auth.credentials import Credentials


credential: Credentials | None = None
lock = asyncio.Lock()


async def get_credential() -> Credentials:
    global credential
    async with lock:
        if credential is None:
            credential, _ = google.auth.default() # type: ignore
    return credential # type: ignore

